/******************************************************************* 
 * This is the code segment for MemBIST inside Memory Service Unit *
 * This only includes the parts of the MSU state machine which are *
 * relevant for MemBIST operation.                                 *
 * This does not include register definitions and the sequential   *
 * section.                                                        *
 *******************************************************************/

      MS_STATE_MBIST_EXD_RET0: begin
        if(!MS_MEMBIST_EXD_RET_IMPLEMENTED) begin
          state_nxt         = MS_STATE_IDLE;
          status_result_nxt = MS_RESULT_NOT_IMPL;
          update_status     = 1'b1;
          ctrl_op_nxt       = MS_CTRL_OP_NONE;
          update_ctrl       = 1'b1;
        end
        else if(ctrl_op_r != MS_CTRL_OP_MBIST_EXD_RET0) begin
          //Cancel BIST operation
          if(master_strans_r==SAMMBA_STRANS_IDLE) begin
            state_nxt         = MS_STATE_IDLE;
            status_result_nxt = MS_RESULT_CANCELED;
            update_status     = 1'b1;
          end
          else if(master_sresp==SAMMBA_SRESP_WAIT) begin
            master_strans_nxt = SAMMBA_STRANS_CANCEL;
          end
          else begin
            master_strans_nxt = SAMMBA_STRANS_IDLE;
          end
        end // if (ctrl_op_r != CTRL_OP_MBIST)
        else if(master_sresp==SAMMBA_SRESP_ERROR ||
                master_sresp==SAMMBA_SRESP_NAK) begin
          //Unaccessible memory
          master_strans_nxt = SAMMBA_STRANS_IDLE;
          state_nxt         = MS_STATE_IDLE;
          status_result_nxt = MS_RESULT_BUS_ERR;
          update_status     = 1'b1;
          ctrl_op_nxt       = MS_CTRL_OP_NONE;
          update_ctrl       = 1'b1;
          master_strans_nxt = SAMMBA_STRANS_IDLE;
        end
        else if(master_strans_r==SAMMBA_STRANS_IDLE || 
                master_sresp==SAMMBA_SRESP_DONE) begin

          //Put read data into DATA register
          data_nxt    = master_srdata;
          update_data = 1'b1;
          /*
           *
           * Do memory data retention BIST - phase 1
           * 
           * Registers used:
           * addrhi_r:      Common SAB address area
           * addrlo_r:      current SAB address
           * length_r:      Remaining bytes in current run
           * tail_r:        Original length
           * stored_data_r: Start address (end address after changing direction)
           * index_r:       Memory operation counter
           * data_r:        SAB read data register
           * master_data_r: SAB write data register
           * 
           * DRF algorithm used:
           * - Phase 1: write 0, read 0 (by word, ascending)
           * - Phase 2: read 0, write 1, read 1 (by word, ascending)
           * - Phase 3: read 1 (by word, ascending)
           * 
           * If we are for example testing for 1 ms data retention, there should be at least
           * 1 ms of delay between the phases. In the meantime, the tester/test bench can do
           * some other useful work.
           * 
           * Therefore the 3 phases are implemented separately and should be called by 
           * separate commands. Make sure to call the 3 phases in the correct order.
           * 
           * This is the state machine for phase 1
           * 
           */
          
          case(index_r)
            'd0: begin
              index_nxt         = 'd1;
              master_strans_nxt = SAMMBA_STRANS_READ;
            end

            'd1: begin
              if (length_r != 'd0) begin
                addrlo_nxt          = addrlo_r + 'd1;
                update_addrlo       = 1'b1;
                master_data_nxt     = 32'h00000000;
                master_strans_nxt   = SAMMBA_STRANS_WRITE;
              end
              else begin
                master_strans_nxt   = SAMMBA_STRANS_IDLE;
              end
              index_nxt = 'd2;
            end

            'd2: begin
              if (data_r != 32'h00000000) begin
                //Error detected; stop
                state_nxt         = MS_STATE_IDLE;
                master_strans_nxt = SAMMBA_STRANS_IDLE;
                status_result_nxt = MS_RESULT_FAILED;
                update_status     = 1'b1;
                ctrl_op_nxt       = MS_CTRL_OP_NONE;
                update_ctrl       = 1'b1;
                //Store bist state and bit index in data for debugging
                data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                update_data       = 1'b1;
              end // if (data_r != 32'hFFFFFFFF)
              else begin
                if (length_r == 'd0) begin
                  state_nxt         = MS_STATE_IDLE;
                  status_result_nxt = MS_RESULT_DONE;
                  update_status     = 1'b1;
                  ctrl_op_nxt       = MS_CTRL_OP_NONE;
                  update_ctrl       = 1'b1;
                end
                else begin
                  length_nxt        = length_r - 'd1;
                  update_length     = 1'b1;
                  master_strans_nxt = SAMMBA_STRANS_READ;
                  index_nxt         = 'd1;
                end // else: !if(length_r == 'd0)
              end // else: !if(data_r != 32'hFFFFFFFF)
            end // case: 'd2

            default: begin // shouldn't happen
              state_nxt         = MS_STATE_IDLE;
              status_result_nxt = MS_RESULT_FAILED;
              update_status     = 1'b1;
              ctrl_op_nxt       = MS_CTRL_OP_NONE;
              update_ctrl       = 1'b1;
              master_strans_nxt = SAMMBA_STRANS_IDLE;
              $display("Unexpected state reached in MemBIST, something is not right!");
            end
          endcase // case (index_r)
        end
      end // case: MS_STATE_MBIST_EXD_RET0

      MS_STATE_MBIST_EXD_RET1: begin
        if(!MS_MEMBIST_EXD_RET_IMPLEMENTED) begin
          state_nxt         = MS_STATE_IDLE;
          status_result_nxt = MS_RESULT_NOT_IMPL;
          update_status     = 1'b1;
          ctrl_op_nxt       = MS_CTRL_OP_NONE;
          update_ctrl       = 1'b1;
        end
        else if(ctrl_op_r != MS_CTRL_OP_MBIST_EXD_RET1) begin
          //Cancel BIST operation
          if(master_strans_r==SAMMBA_STRANS_IDLE) begin
            state_nxt         = MS_STATE_IDLE;
            status_result_nxt = MS_RESULT_CANCELED;
            update_status     = 1'b1;
          end
          else if(master_sresp==SAMMBA_SRESP_WAIT) begin
            master_strans_nxt = SAMMBA_STRANS_CANCEL;
          end
          else begin
            master_strans_nxt = SAMMBA_STRANS_IDLE;
          end
        end // if (ctrl_op_r != CTRL_OP_MBIST)
        else if(master_sresp==SAMMBA_SRESP_ERROR ||
                master_sresp==SAMMBA_SRESP_NAK) begin
          //Unaccessible memory
          master_strans_nxt = SAMMBA_STRANS_IDLE;
          state_nxt         = MS_STATE_IDLE;
          status_result_nxt = MS_RESULT_BUS_ERR;
          update_status     = 1'b1;
          ctrl_op_nxt       = MS_CTRL_OP_NONE;
          update_ctrl       = 1'b1;
          master_strans_nxt = SAMMBA_STRANS_IDLE;
        end
        else if(master_strans_r==SAMMBA_STRANS_IDLE || 
                master_sresp==SAMMBA_SRESP_DONE) begin

          //Put read data into DATA register
          data_nxt    = master_srdata;
          update_data = 1'b1;
          /*
           *
           * Do memory data retention BIST - phase 2
           * 
           * Registers used:
           * addrhi_r:      Common SAB address area
           * addrlo_r:      current SAB address
           * length_r:      Remaining bytes in current run
           * tail_r:        Original length
           * stored_data_r: Start address (end address after changing direction)
           * index_r:       Memory operation counter
           * data_r:        SAB read data register
           * master_data_r: SAB write data register
           * 
           * DRF algorithm used:
           * - Phase 1: write 0, read 0 (by word, ascending)
           * - Phase 2: read 0, write 1, read 1 (by word, ascending)
           * - Phase 3: read 1 (by word, ascending)
           * 
           * If we are for example testing for 1 ms data retention, there should be at least
           * 1 ms of delay between the phases. In the meantime, the tester/test bench can do
           * some other useful work.
           * 
           * Therefore the 3 phases are implemented separately and should be called by 
           * separate commands. Make sure to call the 3 phases in the correct order.
           * 
           * This is the state machine for phase 2
           * 
           */
          
          case(index_r)
            'd0: begin
              index_nxt         = 'd1;
              master_strans_nxt = SAMMBA_STRANS_WRITE;
              master_data_nxt   = 32'hFFFFFFFF;
            end

            'd1: begin
              if (data_r != 32'h00000000) begin
                //Error detected; stop
                state_nxt         = MS_STATE_IDLE;
                master_strans_nxt = SAMMBA_STRANS_IDLE;
                status_result_nxt = MS_RESULT_FAILED;
                update_status     = 1'b1;
                ctrl_op_nxt       = MS_CTRL_OP_NONE;
                update_ctrl       = 1'b1;
                //Store bist state and bit index in data for debugging
                data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                update_data       = 1'b1;
              end // if (data_r != 32'h00000000)
              else begin
                master_strans_nxt   = SAMMBA_STRANS_READ;
                index_nxt = 'd2;
              end
            end

            'd2: begin
              if (length_r != 'd0) begin
                addrlo_nxt          = addrlo_r + 'd1;
                update_addrlo       = 1'b1;
                master_strans_nxt   = SAMMBA_STRANS_READ;
              end
              else begin
                master_strans_nxt   = SAMMBA_STRANS_IDLE;
              end
              index_nxt           = 'd3;
            end // case: 'd2

            'd3: begin
              //Test read data
              if (data_r != 32'hFFFFFFFF) begin
                //Error detected; stop
                state_nxt         = MS_STATE_IDLE;
                master_strans_nxt = SAMMBA_STRANS_IDLE;
                status_result_nxt = MS_RESULT_FAILED;
                update_status     = 1'b1;
                ctrl_op_nxt       = MS_CTRL_OP_NONE;
                update_ctrl       = 1'b1;
                //Store bist state and bit index in data for debugging
                data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                update_data       = 1'b1;
              end // if (data_r != 32'hFFFFFFFF)
              else begin
                if (length_r == 'd0) begin
                  state_nxt         = MS_STATE_IDLE;
                  status_result_nxt = MS_RESULT_DONE;
                  update_status     = 1'b1;
                  ctrl_op_nxt       = MS_CTRL_OP_NONE;
                  update_ctrl       = 1'b1;
                end
                else begin
                  length_nxt      = length_r - 'd1;
                  update_length   = 1'b1;
                  master_strans_nxt = SAMMBA_STRANS_WRITE;
                  master_data_nxt   = 32'hFFFFFFFF;
                  index_nxt         = 'd1;
                end
              end // else: !if(data_r != 32'hFFFFFFFF)                  
            end // case: 'd3

            default: begin // shouldn't happen
              state_nxt         = MS_STATE_IDLE;
              status_result_nxt = MS_RESULT_FAILED;
              update_status     = 1'b1;
              ctrl_op_nxt       = MS_CTRL_OP_NONE;
              update_ctrl       = 1'b1;
              master_strans_nxt = SAMMBA_STRANS_IDLE;
              $display("Unexpected state reached in MemBIST, something is not right!");
            end
          endcase // case (index_r)
        end
      end // case: MS_STATE_MBIST_EXD_RET1
      
      MS_STATE_MBIST_EXD_RET2: begin
        if(!MS_MEMBIST_EXD_RET_IMPLEMENTED) begin
          state_nxt         = MS_STATE_IDLE;
          status_result_nxt = MS_RESULT_NOT_IMPL;
          update_status     = 1'b1;
          ctrl_op_nxt       = MS_CTRL_OP_NONE;
          update_ctrl       = 1'b1;
        end
        else if(ctrl_op_r != MS_CTRL_OP_MBIST_EXD_RET2) begin
          //Cancel BIST operation
          if(master_strans_r==SAMMBA_STRANS_IDLE) begin
            state_nxt         = MS_STATE_IDLE;
            status_result_nxt = MS_RESULT_CANCELED;
            update_status     = 1'b1;
          end
          else if(master_sresp==SAMMBA_SRESP_WAIT) begin
            master_strans_nxt = SAMMBA_STRANS_CANCEL;
          end
          else begin
            master_strans_nxt = SAMMBA_STRANS_IDLE;
          end
        end // if (ctrl_op_r != CTRL_OP_MBIST)
        else if(master_sresp==SAMMBA_SRESP_ERROR ||
                master_sresp==SAMMBA_SRESP_NAK) begin
          //Unaccessible memory
          master_strans_nxt = SAMMBA_STRANS_IDLE;
          state_nxt         = MS_STATE_IDLE;
          status_result_nxt = MS_RESULT_BUS_ERR;
          update_status     = 1'b1;
          ctrl_op_nxt       = MS_CTRL_OP_NONE;
          update_ctrl       = 1'b1;
          master_strans_nxt = SAMMBA_STRANS_IDLE;
        end
        else if(master_strans_r==SAMMBA_STRANS_IDLE || 
                master_sresp==SAMMBA_SRESP_DONE) begin

          //Put read data into DATA register
          data_nxt    = master_srdata;
          update_data = 1'b1;
          /*
           *
           * Do memory data retention BIST - phase 3
           * 
           * Registers used:
           * addrhi_r:      Common SAB address area
           * addrlo_r:      current SAB address
           * length_r:      Remaining bytes in current run
           * tail_r:        Original length
           * stored_data_r: Start address (end address after changing direction)
           * index_r:       Memory operation counter
           * data_r:        SAB read data register
           * master_data_r: SAB write data register
           * 
           * DRF algorithm used:
           * - Phase 1: write 0, read 0 (by word, ascending)
           * - Phase 2: read 0, write 1, read 1 (by word, ascending)
           * - Phase 3: read 1 (by word, ascending)
           * 
           * If we are for example testing for 1 ms data retention, there should be at least
           * 1 ms of delay between the phases. In the meantime, the tester/test bench can do
           * some other useful work.
           * 
           * Therefore the 3 phases are implemented separately and should be called by 
           * separate commands. Make sure to call the 3 phases in the correct order.
           * 
           * This is the state machine for phase 3
           * 
           */
          if (data_r != 32'hFFFFFFFF) begin
            //Error detected; stop
            state_nxt         = MS_STATE_IDLE;
            master_strans_nxt = SAMMBA_STRANS_IDLE;
            status_result_nxt = MS_RESULT_FAILED;
            update_status     = 1'b1;
            ctrl_op_nxt       = MS_CTRL_OP_NONE;
            update_ctrl       = 1'b1;
            //Store bist state and bit index in data for debugging
            data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
            update_data       = 1'b1;
          end // if (data_r != 32'h00000000)
          else begin
            if (length_r == 'd0) begin
              state_nxt         = MS_STATE_IDLE;
              status_result_nxt = MS_RESULT_DONE;
              update_status     = 1'b1;
              ctrl_op_nxt       = MS_CTRL_OP_NONE;
              update_ctrl       = 1'b1;
            end
            else begin
              // Go to next word
              addrlo_nxt      = addrlo_r + 'd1;
              update_addrlo   = 1'b1;
              length_nxt      = length_r - 'd1;
              update_length   = 1'b1;
              master_strans_nxt = SAMMBA_STRANS_READ;
            end
          end // else: !if(data_r != 32'hFFFFFFFF)
        end
      end // case: MS_STATE_MBIST_EXD_RET2
      

      MS_STATE_MBIST_EXD_INTRAWORD_FILL0,
      MS_STATE_MBIST_EXD_INTRAWORD_MARCH0,
      MS_STATE_MBIST_EXD_INTRAWORD_MARCH1,
      MS_STATE_MBIST_EXD_INTRAWORD_MARCH2,
      MS_STATE_MBIST_EXD_INTRAWORD_MARCH3,
      MS_STATE_MBIST_EXD_INTRAWORD_MARCH4,
      MS_STATE_MBIST_EXD_INTRAWORD_MARCH5,
      MS_STATE_MBIST_EXD_INTRAWORD_MARCH6,
      MS_STATE_MBIST_EXD_INTRAWORD_MARCH7,
      MS_STATE_MBIST_EXD_INTERWORD_MARCH : begin
        if(!MS_MEMBIST_EXD_IMPLEMENTED) begin
          state_nxt         = MS_STATE_IDLE;
          status_result_nxt = MS_RESULT_NOT_IMPL;
          update_status     = 1'b1;
          ctrl_op_nxt       = MS_CTRL_OP_NONE;
          update_ctrl       = 1'b1;
        end
        else if(ctrl_op_r != MS_CTRL_OP_MBIST_EXD) begin
          //Cancel BIST operation
          if(master_strans_r==SAMMBA_STRANS_IDLE) begin
            state_nxt         = MS_STATE_IDLE;
            status_result_nxt = MS_RESULT_CANCELED;
            update_status     = 1'b1;
          end
          else if(master_sresp==SAMMBA_SRESP_WAIT) begin
            master_strans_nxt = SAMMBA_STRANS_CANCEL;
          end
          else begin
            master_strans_nxt = SAMMBA_STRANS_IDLE;
          end
        end // if (ctrl_op_r != CTRL_OP_MBIST_EXD)
        else if(master_sresp==SAMMBA_SRESP_ERROR ||
                master_sresp==SAMMBA_SRESP_NAK) begin
          //Unaccessible memory
          master_strans_nxt = SAMMBA_STRANS_IDLE;
          state_nxt         = MS_STATE_IDLE;
          status_result_nxt = MS_RESULT_BUS_ERR;
          update_status     = 1'b1;
          ctrl_op_nxt       = MS_CTRL_OP_NONE;
          update_ctrl       = 1'b1;
          master_strans_nxt = SAMMBA_STRANS_IDLE;
        end
        else if(master_strans_r==SAMMBA_STRANS_IDLE || 
                master_sresp==SAMMBA_SRESP_DONE) begin

          //Put read data into DATA register
          data_nxt    = master_srdata;
          update_data = 1'b1;
          /*
           *
           * Do memory BIST
           * 
           * Registers used:
           * addrhi_r:      Common SAB address area
           * addrlo_r:      current SAB address
           * length_r:      Remaining bytes in current run
           * tail_r:        Original length
           * stored_data_r: Start address (end address after changing direction)
           * index_r:       indicates which step in each march element is being performed
           * data_r:        SAB read data register
           * master_data_r: SAB write data register
           * 
           * The algorithm used is a combination of "March AB" and "March LR" extended to
           * Word Oriented Memory (WOM) using crCFdst fault model
           * 
           * All operations are done WORD-by-WORD
           * 
           * - INTRAWORD_FILL0  : Fill entire memory with 0xffffffff, ascending
           * - INTRAWORD_MARCH0 : read 0xffffffff, write 0x00000000, read 0x00000000,
           *                      write 0x00000000, read 0x00000000, descending
           * - INTRAWORD_MARCH1 : read 0x00000000, write 0xffffffff, read 0xffffffff,
           *                      write 0xffffffff, read 0xffffffff, descending
           * - INTRAWORD_MARCH2 : read 0xffffffff, write 0x00000000, read 0x00000000,
           *                      write 0x00000000, read 0x00000000, ascending
           * - INTRAWORD_MARCH3 : read 0x00000000, write 0xffffffff, read 0xffffffff,
           *                      write 0xffffffff, read 0xffffffff, ascending
           * - INTRAWORD_MARCH4 : read 0xffffffff, write 0x00000000,
           *                    : read 0x00000000, write 0xffffffff, ascending
           * - INTRAWORD_MARCH5 : read 0xffffffff, write 0x00000000, ascending
           * - INTRAWORD_MARCH6 : read 0x00000000, write 0xffffffff,
           *                      read 0xffffffff, write 0x00000000, ascending
           * - INTRAWORD_MARCH7 : read 0x00000000, ascending
           * - INTERWORD_MARCH  : write 0xffffffff, read 0xffffffff, read 0xffffffff,
           *                      write 0x00000000, read 0x00000000, read 0x00000000,
           *                      write 0x24924924,
           *                      write 0xdb6db6db, read 0xdb6db6db, read 0xdb6db6db,
           *                      write 0x24924924, read 0x24924924, read 0x24924924,
           *                      write 0x42942942,
           *                      write 0xb6db6db6, read 0xb6db6db6, read 0xb6db6db6,
           *                      write 0x42942942, read 0x42942942, read 0x42942942,
           *                      write 0x6db6db6d,
           *                      write 0x92492492, read 0x92492492, read 0x92492492,
           *                      write 0x6db6db6d, read 0x6db6db6d, read 0x6db6db6d,
           *                      descending
           */
          
          case(state_r)

            //Fill memory with 0xffffffff
            MS_STATE_MBIST_EXD_INTRAWORD_FILL0: begin
              if(length_r=='d0) begin
                //Go to next pass: MARCH0
                state_nxt         = MS_STATE_MBIST_EXD_INTRAWORD_MARCH0;
                length_nxt        = tail_r;
                update_length     = 1'b1;
                index_nxt         = 'd0;
                master_strans_nxt = SAMMBA_STRANS_READ;
              end
              else begin
                //Go to next word
                addrlo_nxt        = addrlo_r + 'd1;
                update_addrlo     = 1'b1;
                length_nxt        = length_r - 'd1;
                update_length     = 1'b1;
              end // else: !if(length_r=='d0)
            end // case: MS_STATE_MBIST_EXD_INTRAWORD_FILL0
            
            MS_STATE_MBIST_EXD_INTRAWORD_MARCH0: begin
              case(index_r)
                'd0: begin
                  master_data_nxt   = 32'h00000000;
                  master_strans_nxt = SAMMBA_STRANS_WRITE;
                  index_nxt = 'd1;                                    
                end // case: 'd0
                
                'd1: begin
                  //Test read data
                  if (data_r != 32'hFFFFFFFF) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'hFFFFFFFF)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_READ;
                    index_nxt         = 'd2;
                  end // else: !if(data_r != 32'hFFFFFFFF)
                end // case: 'd1

                'd2: begin
                  master_data_nxt     = 32'h00000000;
                  master_strans_nxt   = SAMMBA_STRANS_WRITE;
                  index_nxt           = 'd3;
                end // case: 'd2

                'd3: begin
                  //Test read data
                  if (data_r != 32'h00000000) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'h00000000)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_READ;
                    index_nxt         = 'd4;
                  end // else: !if(data_r != 32'h000000000)                  
                end // case: 'd3

                'd4: begin
                  if (length_r == 'd0) begin
                    // Go to pass MARCH1
                    addrlo_nxt        = stored_addr_r + tail_r;
                    update_addrlo     = 1'b1;
                  end
                  else begin
                    // Go to next word
                    addrlo_nxt        = addrlo_r - 'd1;
                    update_addrlo     = 1'b1;
                  end
                  index_nxt = 'd5;
                  master_strans_nxt = SAMMBA_STRANS_READ;
                end //case 'd4

                'd5: begin
                  if (data_r != 32'h00000000) begin
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // else: !if(data_r != 32'h000000000)
                  else if (length_r == 'd0) begin
                    // Go to pass MARCH1
                    state_nxt         = MS_STATE_MBIST_EXD_INTRAWORD_MARCH1;
                    index_nxt         = 'd1;
                    master_data_nxt   = 32'hFFFFFFFF;
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    length_nxt        = tail_r;
                    update_length     = 1'b1;
                  end
                  else begin
                    // Go to next word
                    master_data_nxt   = 32'h00000000;
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    index_nxt         = 'd1;
                    length_nxt        = length_r - 'd1;
                    update_length     = 1'b1;
                  end
                end //case:'d5

                default: begin // shouldn't happen
                  state_nxt         = MS_STATE_IDLE;
                  status_result_nxt = MS_RESULT_FAILED;
                  update_status     = 1'b1;
                  ctrl_op_nxt       = MS_CTRL_OP_NONE;
                  update_ctrl       = 1'b1;
                  master_strans_nxt = SAMMBA_STRANS_IDLE;
                  $display("Unexpected state reached in MemBIST, something is not right!");
                end

              endcase // case(index_r)
            end //case: MS_STATE_MBIST_EXD_MARCH0

            MS_STATE_MBIST_EXD_INTRAWORD_MARCH1: begin
              case(index_r)
                'd1: begin
                  //Test read data
                  if (data_r != 32'h00000000) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'h00000000)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_READ;
                    index_nxt         = 'd2;
                  end // else: !if(data_r != 32'h00000000)
                end // case: 'd0

                'd2: begin
                  master_data_nxt     = 32'hFFFFFFFF;
                  master_strans_nxt   = SAMMBA_STRANS_WRITE;
                  index_nxt           = 'd3;
                end // case: 'd1

                'd3: begin
                  //Test read data
                  if (data_r != 32'hFFFFFFFF) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'hFFFFFFFF)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_READ;
                    index_nxt         = 'd4;
                  end // else: !if(data_r != 32'hFFFFFFFF)                  
                end // case: 'd2

                'd4: begin
                  if (length_r != 'd0) begin
                    // Go to next word
                    addrlo_nxt        = addrlo_r - 'd1;
                    update_addrlo     = 1'b1;
                  end
                  index_nxt = 'd5;
                  master_strans_nxt = SAMMBA_STRANS_READ;
                end //case 'd3

                'd5: begin
                  if (data_r != 32'hFFFFFFFF) begin
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'hFFFFFFFF)
                  else if (length_r == 'd0) begin
                    // Go to pass MARCH2
                    state_nxt         = MS_STATE_MBIST_EXD_INTRAWORD_MARCH2;
                    index_nxt         = 'd1;
                    master_data_nxt   = 32'h00000000;
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    length_nxt        = tail_r;
                    update_length     = 1'b1;
                  end
                  else begin
                    // Go to next word
                    master_data_nxt   = 32'hFFFFFFFF;
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    index_nxt         = 'd1;
                    length_nxt        = length_r - 'd1;
                    update_length     = 1'b1;
                  end
                end //case:'d4

                default: begin // shouldn't happen
                  state_nxt         = MS_STATE_IDLE;
                  status_result_nxt = MS_RESULT_FAILED;
                  update_status     = 1'b1;
                  ctrl_op_nxt       = MS_CTRL_OP_NONE;
                  update_ctrl       = 1'b1;
                  master_strans_nxt = SAMMBA_STRANS_IDLE;
                  $display("Unexpected state reached in MemBIST, something is not right!");
                end

              endcase // case(index_r)
            end //case: MS_STATE_MBIST_EXD_MARCH1
            
            MS_STATE_MBIST_EXD_INTRAWORD_MARCH2: begin
              case (index_r)
                'd1: begin
                  //Test read data
                  if (data_r != 32'hFFFFFFFF) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'hFFFFFFFF)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_READ;
                    index_nxt         = 'd2;
                  end // else: !if(data_r != 32'hFFFFFFFF)
                end // case: 'd1

                'd2: begin
                  master_data_nxt     = 32'h00000000;
                  master_strans_nxt   = SAMMBA_STRANS_WRITE;
                  index_nxt           = 'd3;
                end // case: 'd2

                'd3: begin
                  //Test read data
                  if (data_r != 32'h00000000) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'h00000000)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_READ;
                    index_nxt         = 'd4;
                  end // else: !if(data_r != 32'h000000000)                  
                end // case: 'd3

                'd4: begin
                  if (length_r == 'd0) begin
                    // Go to pass MARCH3
                    addrlo_nxt        = stored_addr_r;
                    update_addrlo     = 1'b1;
                  end
                  else begin
                    // Go to next word
                    addrlo_nxt        = addrlo_r + 'd1;
                    update_addrlo     = 1'b1;
                  end
                  index_nxt = 'd5;
                  master_strans_nxt = SAMMBA_STRANS_READ;
                end //case 'd4

                'd5: begin
                  if (data_r != 32'h00000000) begin
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // else: !if(data_r != 32'h000000000)
                  else if (length_r == 'd0) begin
                    // Go to pass MARCH1
                    state_nxt         = MS_STATE_MBIST_EXD_INTRAWORD_MARCH3;
                    index_nxt         = 'd1;
                    master_data_nxt   = 32'hFFFFFFFF;
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    length_nxt        = tail_r;
                    update_length     = 1'b1;
                  end
                  else begin
                    // Go to next word
                    master_data_nxt   = 32'h00000000;
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    index_nxt         = 'd1;
                    length_nxt        = length_r - 'd1;
                    update_length     = 1'b1;
                  end
                end //case:'d5

                default: begin // shouldn't happen
                  state_nxt         = MS_STATE_IDLE;
                  status_result_nxt = MS_RESULT_FAILED;
                  update_status     = 1'b1;
                  ctrl_op_nxt       = MS_CTRL_OP_NONE;
                  update_ctrl       = 1'b1;
                  master_strans_nxt = SAMMBA_STRANS_IDLE;
                  $display("Unexpected state reached in MemBIST, something is not right!");
                end

              endcase // case(index_r)
            end //case: MS_STATE_MBIST_EXD_MARCH2

            MS_STATE_MBIST_EXD_INTRAWORD_MARCH3: begin
              case(index_r)
                'd1: begin
                  //Test read data
                  if (data_r != 32'h00000000) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'h00000000)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_READ;
                    index_nxt         = 'd2;
                  end // else: !if(data_r != 32'h00000000)
                end // case: 'd0

                'd2: begin
                  master_data_nxt     = 32'hFFFFFFFF;
                  master_strans_nxt   = SAMMBA_STRANS_WRITE;
                  index_nxt           = 'd3;
                end // case: 'd1

                'd3: begin
                  //Test read data
                  if (data_r != 32'hFFFFFFFF) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'hFFFFFFFF)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_READ;
                    index_nxt         = 'd4;
                  end // else: !if(data_r != 32'hFFFFFFFF)                  
                end // case: 'd2

                'd4: begin
                  if (length_r == 'd0) begin
                    // Go to pass MARCH4
                    addrlo_nxt        = stored_addr_r;
                    update_addrlo     = 1'b1;
                  end
                  else begin
                    // Go to next word
                    addrlo_nxt        = addrlo_r + 'd1;
                    update_addrlo     = 1'b1;
                  end
                  index_nxt = 'd5;
                  master_strans_nxt = SAMMBA_STRANS_READ;
                end //case 'd3

                'd5: begin
                  if (data_r != 32'hFFFFFFFF) begin
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // else: !if(data_r != 32'h000000000)
                  else if (length_r == 'd0) begin
                    // Go to pass MARCH4
                    state_nxt         = MS_STATE_MBIST_EXD_INTRAWORD_MARCH4;
                    index_nxt         = 'd1;
                    master_data_nxt   = 32'h00000000;
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    length_nxt        = tail_r;
                    update_length     = 1'b1;
                  end
                  else begin
                    // Go to next word
                    master_data_nxt   = 32'hFFFFFFFF;
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    index_nxt         = 'd1;
                    length_nxt        = length_r - 'd1;
                    update_length     = 1'b1;
                  end
                end //case:'d4

                default: begin // shouldn't happen
                  state_nxt         = MS_STATE_IDLE;
                  status_result_nxt = MS_RESULT_FAILED;
                  update_status     = 1'b1;
                  ctrl_op_nxt       = MS_CTRL_OP_NONE;
                  update_ctrl       = 1'b1;
                  master_strans_nxt = SAMMBA_STRANS_IDLE;
                  $display("Unexpected state reached in MemBIST, something is not right!");
                end

              endcase // case(index_r)
            end //case: MS_STATE_MBIST_EXD_MARCH3           
            
            MS_STATE_MBIST_EXD_INTRAWORD_MARCH4: begin
              case (index_r)
                'd1: begin
                  //Test read data
                  if (data_r != 32'hFFFFFFFF) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'hFFFFFFFF)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_READ;
                    index_nxt         = 'd2;
                  end // else: !if(data_r != 32'hFFFFFFFF)
                end // case: 'd1

                'd2: begin
                  master_data_nxt     = 32'hFFFFFFFF;
                  master_strans_nxt   = SAMMBA_STRANS_WRITE;
                  index_nxt           = 'd3;
                end // case: 'd2

                'd3: begin
                  //Test read data
                  if (data_r != 32'h00000000) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'h00000000)
                  else begin
                    if (length_r == 'd0) begin
                      // Go to pass MARCH5
                      addrlo_nxt      = stored_addr_r;
                      update_addrlo   = 1'b1;
                    end
                    else begin
                      // Go to next word
                      addrlo_nxt      = addrlo_r + 'd1;
                      update_addrlo   = 1'b1;
                    end
                    master_strans_nxt = SAMMBA_STRANS_READ;
                    index_nxt         = 'd4;
                  end // else: !if(data_r != 32'h000000000)                  
                end // case: 'd3

                'd4: begin
                  if (length_r == 'd0) begin
                    // Go to pass MARCH5
                    length_nxt        = tail_r;
                    update_length     = 1'b1;
                    state_nxt         = MS_STATE_MBIST_EXD_INTRAWORD_MARCH5;
                  end
                  else begin
                    length_nxt      = length_r - 'd1;
                    update_length   = 1'b1;
                  end
                  index_nxt         = 'd1;
                  master_data_nxt   = 32'h00000000;
                  master_strans_nxt = SAMMBA_STRANS_WRITE;
                end //case 'd4

                default: begin // shouldn't happen
                  state_nxt         = MS_STATE_IDLE;
                  status_result_nxt = MS_RESULT_FAILED;
                  update_status     = 1'b1;
                  ctrl_op_nxt       = MS_CTRL_OP_NONE;
                  update_ctrl       = 1'b1;
                  master_strans_nxt = SAMMBA_STRANS_IDLE;
                  $display("Unexpected state reached in MemBIST, something is not right!");
                end

              endcase // case (index_r)
            end // case: MS_STATE_MBIST_EXD_MARCH4

            MS_STATE_MBIST_EXD_INTRAWORD_MARCH5: begin
              case (index_r)
                'd1: begin
                  //Test read data
                  if (data_r != 32'hFFFFFFFF) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'hFFFFFFFF)
                  else begin
                    if (length_r == 'd0) begin
                      // Go to pass MARCH6
                      addrlo_nxt      = stored_addr_r;
                      update_addrlo   = 1'b1;
                    end
                    else begin
                      // Go to next word
                      addrlo_nxt      = addrlo_r + 'd1;
                      update_addrlo   = 1'b1;
                    end
                    master_strans_nxt = SAMMBA_STRANS_READ;
                    index_nxt         = 'd2;
                  end // else: !if(data_r != 32'hFFFFFFFF)
                end // case: 'd1

                'd2: begin
                  if (length_r == 'd0) begin
                    // Go to pass MARCH6
                    length_nxt        = tail_r;
                    update_length     = 1'b1;
                    state_nxt         = MS_STATE_MBIST_EXD_INTRAWORD_MARCH6;
                    master_data_nxt   = 32'hFFFFFFFF;
                  end
                  else begin
                    master_data_nxt   = 32'h00000000;
                    length_nxt      = length_r - 'd1;
                    update_length   = 1'b1;
                  end
                  master_strans_nxt = SAMMBA_STRANS_WRITE;
                  index_nxt         = 'd1;
                end //case 'd2

                default: begin // shouldn't happen
                  state_nxt         = MS_STATE_IDLE;
                  status_result_nxt = MS_RESULT_FAILED;
                  update_status     = 1'b1;
                  ctrl_op_nxt       = MS_CTRL_OP_NONE;
                  update_ctrl       = 1'b1;
                  master_strans_nxt = SAMMBA_STRANS_IDLE;
                  $display("Unexpected state reached in MemBIST, something is not right!");
                end

              endcase // case (index_r)
            end // case: MS_STATE_MBIST_EXD_INTRAWORD_MARCH5

            MS_STATE_MBIST_EXD_INTRAWORD_MARCH6: begin
              case (index_r)
                'd1: begin
                  if (data_r != 32'h00000000) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'h00000000)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_READ;
                    index_nxt         = 'd2;
                  end // else: !if(data_r != 32'hFFFFFFFF)
                end // case: 'd1

                'd2: begin
                  master_data_nxt     = 32'h00000000;
                  master_strans_nxt   = SAMMBA_STRANS_WRITE;
                  index_nxt           = 'd3;
                end // case: 'd2

                'd3: begin
                  //Test read data
                  if (data_r != 32'hFFFFFFFF) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'hFFFFFFFF)
                  else begin
                    if (length_r == 'd0) begin
                      // Go to pass MARCH7
                      addrlo_nxt        = stored_addr_r;
                      update_addrlo     = 1'b1;
                    end
                    else begin
                      // Go to next word
                      addrlo_nxt      = addrlo_r + 'd1;
                      update_addrlo   = 1'b1;
                    end
                    master_strans_nxt = SAMMBA_STRANS_READ;
                    index_nxt         = 'd4;
                  end // else: !if(data_r != 32'hFFFFFFFF)                  
                end // case: 'd2

                'd4: begin
                  if (length_r == 'd0) begin
                    // Go to pass MARCH7
                    length_nxt        = tail_r;
                    update_length     = 1'b1;
                    state_nxt         = MS_STATE_MBIST_EXD_INTRAWORD_MARCH7;
                    master_strans_nxt = SAMMBA_STRANS_READ;
                  end
                  else begin
                    length_nxt      = length_r - 'd1;
                    update_length   = 1'b1;
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    master_data_nxt   = 32'hFFFFFFFF;
                  end
                  index_nxt         = 'd1;
                end // case: 'd2

                default: begin // shouldn't happen
                  state_nxt         = MS_STATE_IDLE;
                  status_result_nxt = MS_RESULT_FAILED;
                  update_status     = 1'b1;
                  ctrl_op_nxt       = MS_CTRL_OP_NONE;
                  update_ctrl       = 1'b1;
                  master_strans_nxt = SAMMBA_STRANS_IDLE;
                  $display("Unexpected state reached in MemBIST, something is not right!");
                end

              endcase // case (index_r)
            end // case: MS_STATE_MBIST_EXD_INTRAWORD_MARCH6

            MS_STATE_MBIST_EXD_INTRAWORD_MARCH7: begin
              if (data_r != 32'h00000000) begin
                //Error detected; stop
                state_nxt         = MS_STATE_IDLE;
                master_strans_nxt = SAMMBA_STRANS_IDLE;
                status_result_nxt = MS_RESULT_FAILED;
                update_status     = 1'b1;
                ctrl_op_nxt       = MS_CTRL_OP_NONE;
                update_ctrl       = 1'b1;
                //Store bist state and bit index in data for debugging
                data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                update_data       = 1'b1;
              end // if (data_r != 32'h00000000)
              else begin
                if (length_r == 'd0) begin
                  // Go to pass INTERWORD_MARCH
                  length_nxt        = tail_r;
                  update_length     = 1'b1;
                  master_strans_nxt = SAMMBA_STRANS_WRITE;
                  master_data_nxt   = 32'hFFFFFFFF;
                  index_nxt        = 'd0;
                  state_nxt         = MS_STATE_MBIST_EXD_INTERWORD_MARCH;
                end
                else begin
                  // Go to next word
                  addrlo_nxt      = addrlo_r + 'd1;
                  update_addrlo   = 1'b1;
                  length_nxt      = length_r - 'd1;
                  update_length   = 1'b1;
                  master_strans_nxt = SAMMBA_STRANS_READ;
                end
              end // else: !if(data_r != 32'hFFFFFFFF)
            end // case: MS_STATE_MBIST_EXD_INTRAWORD_MARCH7

            MS_STATE_MBIST_EXD_INTERWORD_MARCH: begin
              case (index_r)
                'd0: begin
                  master_strans_nxt = SAMMBA_STRANS_READ;
                  index_nxt = 'd1;
                end

                'd1: begin
                  master_strans_nxt = SAMMBA_STRANS_READ;
                  index_nxt         = 'd2;
                end

                'd2: begin
                  if (data_r != 32'hFFFFFFFF) begin
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'hFFFFFFFF)
                  else begin
                    master_data_nxt   = 32'h00000000;
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    index_nxt         = 'd3;
                  end // else: !if(data_r != 32'hFFFFFFFF)
                end // case: 'd2

                'd3: begin
                  if (data_r != 32'hFFFFFFFF) begin
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'hFFFFFFFF)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_READ;
                    index_nxt         = 'd4;
                  end // else: !if(data_r != 32'hFFFFFFFF)
                end // case: 'd3

                'd4: begin
                  master_strans_nxt = SAMMBA_STRANS_READ;
                  index_nxt         = 'd5;
                end

                'd5: begin
                  if (data_r != 32'h00000000) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'h00000000)
                  else begin
                    master_data_nxt   = 32'h24924924;
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    index_nxt         = 'd6;
                  end // else: !if(data_r != 32'h00000000)
                end // case: 'd5

                'd6: begin
                  if (data_r != 32'h00000000) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'h00000000)
                  else begin
                    master_data_nxt   = 32'hDB6DB6DB;
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    index_nxt         = 'd7;
                  end // else: !if(data_r != 32'h00000000)                  
                end // case: 'd6

                'd7: begin
                  master_strans_nxt = SAMMBA_STRANS_READ;
                  index_nxt         = 'd8;
                end
                
                'd8: begin
                  master_strans_nxt = SAMMBA_STRANS_READ;
                  index_nxt         = 'd9;
                end
                
                'd9: begin
                  if (data_r != 32'hDB6DB6DB) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'hDB6DB6DB6)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    master_data_nxt   = 32'h24924924;
                    index_nxt         = 'd10;
                  end // else: !if(data_r != 32'hDB6DB6DB6)
                end // case: 'd9

                'd10: begin
                  if (data_r != 32'hDB6DB6DB) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'hDB6DB6DB6)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_READ;
                    index_nxt         = 'd11;
                  end // else: !if(data_r != 32'hDB6DB6DB6)
                end // case: 'd10

                'd11: begin
                  master_strans_nxt = SAMMBA_STRANS_READ;
                  index_nxt         = 'd12;
                end

                'd12: begin
                  if (data_r != 32'h24924924) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'h24924924)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    master_data_nxt   = 32'h42942942;
                    index_nxt         = 'd13;
                  end // else: !if(data_r != 32'h24924924)
                end // case: 'd12

                'd13: begin
                  if (data_r != 32'h24924924) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'h24924924)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    master_data_nxt   = 32'hB6DB6DB6;
                    index_nxt         = 'd14;
                  end // else: !if(data_r != 32'h24924924)
                end // case: 'd13

                'd14: begin
                  master_strans_nxt = SAMMBA_STRANS_READ;
                  index_nxt         = 'd15;
                end
                
                'd15: begin
                  master_strans_nxt = SAMMBA_STRANS_READ;
                  index_nxt         = 'd16;
                end

                'd16: begin
                  if (data_r != 32'hB6DB6DB6) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'hB6DB6DB6D)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    master_data_nxt   = 32'h42942942;
                    index_nxt         = 'd17;
                  end // else: !if(data_r != 32'hB6DB6DB6D)
                end // case: 'd16

                'd17: begin
                  if (data_r != 32'hB6DB6DB6) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'hB6DB6DB6D)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_READ;
                    index_nxt         = 'd18;
                  end // else: !if(data_r != 32'hB6DB6DB6D)
                end // case: 'd17

                'd18: begin
                  master_strans_nxt = SAMMBA_STRANS_READ;
                  index_nxt        = 'd19;
                end

                'd19: begin
                  if (data_r != 32'h42942942) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'h429429429)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    master_data_nxt   = 32'h6DB6DB6D;
                    index_nxt         = 'd20;
                  end // else: !if(data_r != 32'h429429429)
                end // case: 'd19
                
                'd20: begin
                  master_strans_nxt = SAMMBA_STRANS_WRITE;
                  master_data_nxt   = 32'h92492492;
                  index_nxt         = 'd21;
                end

                'd21: begin
                  master_strans_nxt = SAMMBA_STRANS_READ;
                  index_nxt         = 'd22;
                end

                'd22: begin
                  master_strans_nxt = SAMMBA_STRANS_READ;
                  index_nxt         = 'd23;
                end
                
                'd23: begin
                  if (data_r != 32'h92492492) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'h92492492)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_WRITE;
                    master_data_nxt   = 32'h6DB6DB6D;
                    index_nxt         = 'd24;
                  end // else: !if(data_r != 32'h92492492)
                end // case: 'd23

                'd24: begin
                  if (data_r != 32'h92492492) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'h92492492)
                  else begin
                    master_strans_nxt = SAMMBA_STRANS_READ;
                    index_nxt         = 'd25;
                  end // else: !if(data_r != 32'h92492492)
                end // case: 'd24

                'd25: begin
                  master_strans_nxt = SAMMBA_STRANS_READ;
                  index_nxt         = 'd26;
                end

                'd26: begin
                  if (data_r != 32'h6DB6DB6D) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'h92492492)
                  else begin
                    if (length_r == 'd0) begin
                      master_strans_nxt = SAMMBA_STRANS_IDLE;
                    end
                    else begin
                      // Go to next word
                      addrlo_nxt        = addrlo_r - 'd1;
                      update_addrlo     = 1'b1;
                      master_strans_nxt = SAMMBA_STRANS_WRITE;
                      master_data_nxt   = 32'hFFFFFFFF;
                    end
                    index_nxt = 'd27;
                  end // else: !if(data_r != 32'h92492492)
                end // case: 'd26

                'd27: begin
                  if (data_r != 32'h6DB6DB6D) begin
                    //Error detected; stop
                    state_nxt         = MS_STATE_IDLE;
                    master_strans_nxt = SAMMBA_STRANS_IDLE;
                    status_result_nxt = MS_RESULT_FAILED;
                    update_status     = 1'b1;
                    ctrl_op_nxt       = MS_CTRL_OP_NONE;
                    update_ctrl       = 1'b1;
                    //Store bist state and bit index in data for debugging
                    data_nxt          = {state_r[3:0], 2'b00, index_r[4:0]};
                    update_data       = 1'b1;
                  end // if (data_r != 32'h92492492)
                  else begin
                    if (length_r == 'd0) begin
                      state_nxt         = MS_STATE_IDLE;
                      status_result_nxt = MS_RESULT_DONE;
                      update_status     = 1'b1;
                      ctrl_op_nxt       = MS_CTRL_OP_NONE;
                      update_ctrl       = 1'b1;
                      $display("%t, MemBIST Ended Successfully", $time);
                    end
                    else begin
                      length_nxt        = length_r - 'd1;
                      update_length     = 1'b1;
                      master_strans_nxt = SAMMBA_STRANS_READ;
                      index_nxt         = 'd1;
                    end // else: !if(length_r == 'd0)
                      
                  end // else: !if(data_r != 32'h92492492)
                end // case: 'd27

                default: begin // shouldn't happen
                  state_nxt         = MS_STATE_IDLE;
                  status_result_nxt = MS_RESULT_FAILED;
                  update_status     = 1'b1;
                  ctrl_op_nxt       = MS_CTRL_OP_NONE;
                  update_ctrl       = 1'b1;
                  master_strans_nxt = SAMMBA_STRANS_IDLE;
                  $display("Unexpected state reached in MemBIST, something is not right!");
                end
                
              endcase // case (index_r)

            end // case: MS_STATE_MBIST_EXD_INTERWORD_MARCH
            
            
            default: begin //shouldn't happen
              state_nxt         = MS_STATE_IDLE;
              status_result_nxt = MS_RESULT_FAILED;
              update_status     = 1'b1;
              ctrl_op_nxt       = MS_CTRL_OP_NONE;
              update_ctrl       = 1'b1;
              master_strans_nxt = SAMMBA_STRANS_IDLE;
            end
            
          endcase // case(state_r)
        end // case: else...            
      end // case: MS_STATE_MBIST_INIT,...
  
